var isLibExpired = true;

//Set start date manually
var startDate = new Date();

//Set expiry date manually
var expiryDate = new Date();

/**
 * Checks the validity of Library
 * If library is expired, APIs cannot be used
 * Start and expiry dates are manually set
 * */
function initLib(){
	setDate(startDate, 2012, 4, 1, 0, 0, 0);
	setDate(expiryDate, 2012, 5, 30, 23, 59, 59)
	
	var sysDate = new Date();
	if (sysDate == expiryDate){
		console.log("Today is SmartWemo's expiry date");	
	}
	
	if (startDate <= sysDate && sysDate <= expiryDate){
		isLibExpired = false;
	} else {
		isLibExpired = true;
		alert("Sorry, the library you are using has expired..")
	}
	console.log("Library is expired? " + isLibExpired);	
}

/** 
 * Sets the date
 * date Date object
 * yy year
 * mm month
 * dd day
 * hh hour
 * min minutes
 * ss seconds
 * 
 * */
function setDate(date, yy, mm, dd, hh, min, ss) {
	date.setFullYear(yy, mm, dd);
	date.setHours(hh);
	date.setMinutes(min);
	date.setSeconds(ss);	
} 

if (typeof WeMo === "undefined") {
	WeMo = {
        id : null,
		request: {
			className : null,
			methodName : null,
			methodParams: [],
            callbackId: -1,
		},
		libraries: [],
        callbacks: {}
	};
	WeMo.cbResult = {
        NO_RESULT: 0,
        OK: 1,
        CLASS_NOT_FOUND_EXCEPTION: 2,
        ILLEGAL_ACCESS_EXCEPTION: 3,
        INSTANTIATION_EXCEPTION: 4,
        MALFORMED_URL_EXCEPTION: 5,
        IO_EXCEPTION: 6,
        INVALID_ACTION: 7,
        JSON_EXCEPTION: 8,
        ERROR: 9
    };
	// List of loaded libraries
	WeMo.libraries = { main: true };
	// Check if library exists
	WeMo.hasLibrary = function(name) {
		return WeMo.libraries[name];
	};
	// Add new library item
	WeMo.addLibrary = function(name) {
		return WeMo.libraries[name] = true;
	};
	// Send request to server.
	WeMo.sendRequest = function(successCB, errorCB) {
        if (WeMo.id == null) {
            WeMo.id = 1;
        } else {
            WeMo.id++;
        }
        if (successCB || errorCB) {
            var id = WeMo.id;
            WeMo.request.callbackId = id;
            WeMo.callbacks[id] = {
                success: successCB,
                error: errorCB
            }
            //            alert("" + id);
        }
		document.location.href = "ubiq://go";
	}
	// Get request
	WeMo.fetchRequest = function() {
		var request = JSON.stringify(WeMo.request);
        return request;
	}
    
    /**
     * Called by native code when returning successful result from an action.
     *
     * @param callbackId
     * @param args
     *        args.result - WeMo.cbResult
     *        args.message - return value
     *        args.remove - 0 to remove callback, 1 to keep callback in WeMo.callbacks[]
     */
    WeMo.success = function(callbackId, args) {
        if (WeMo.callbacks[callbackId]) {
            
            // If result is to be sent to callback
            if (args.status == WeMo.cbResult.OK) {
                try {
                    if (WeMo.callbacks[callbackId].success) {
                        WeMo.callbacks[callbackId].success(args.message);
                    }
                } catch (e) {
                    console.log("Error in success callback: " + callbackId + " = " + e);
                }
            }
            
            // Clear callback if not expecting any more results
            if (!args.keepCallback) {
                delete WeMo.callbacks[callbackId];
            }
        }
    };
    
    /**
     * Called by native code when returning error result from an action.
     *
     * @param callbackId
     * @param args
     */
    WeMo.error = function(callbackId, args) {
        if (WeMo.callbacks[callbackId]) {
            try {
                if (WeMo.callbacks[callbackId].error) {
                    WeMo.callbacks[callbackId].error(args.message);
                }
            } catch (e) {
                console.log("Error in error callback: " + callbackId + " = " + e);
            }
            
            // Clear callback if not expecting any more results
            if (!args.keepCallback) {
                delete WeMo.callbacks[callbackId];
            }
        }
    };
}

if (!WeMo.hasLibrary("acceleration")) {
	WeMo.addLibrary("acceleration");
	
	Acceleration = function(aX, aY, aZ) {
		this.aX = aX;
		this.aY = aY;
		this.aZ = aZ;
		this.aTimestamp = new Date().getTime();
	}	
    // Set MAX VALUES of Accelerometer here! (Platform dependent)
    Acceleration.MAX_X = 1.05;
    Acceleration.MIN_X = -1.05;
    Acceleration.MAX_Y = 1.05;
    Acceleration.MIN_Y = -1.05;
    Acceleration.MAX_Z = 1.05;
    Acceleration.MIN_Z = -1.05;
}

if(!WeMo.hasLibrary("accelerometer")) {
    WeMo.addLibrary("accelerometer");
    var interval;
    
	Accelerometer = function() {
		this.latestAcceleration = new Acceleration(0, 0, 0);
	}
	
	Accelerometer.startListener = function(successCB, errorCB, params) {
        var errMsg = "";
		if ( isLibExpired == true ) {
			return;
		}        
        if (typeof params.samplingRate === "undefined") {
            errMsg = "Sampling rate is undefined!";
            if (typeof errorCB === "function") {
                errorCB(errMsg);
            }
            console.log(errMsg);
            return;
        } else if (params.samplingRate <= 0 || params.samplingRate > 60) {
            // Limit sampling rate to 1Hz to 60Hz
            errMsg = "Sampling rate should be a value from 1 to 60!";
            if (typeof errorCB === "function") {
                errorCB(errMsg);
            }
            console.log(errMsg);
            return;
        }
        
		WeMo.request = {
            className : "AccelerometerLib",
            methodName : "startReceivingAccelerometerUpdates:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
		
        // Update every 1000 / params.samplingRate ms
		interval = setInterval(function() {
			Accelerometer.getCurrentAcceleration(successCB, errorCB, params);
		}, 1000 / params.samplingRate);
	}
	
	Accelerometer.stopListener = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
		WeMo.request = {
            className : "AccelerometerLib",
            methodName : "stopReceivingAccelerometerUpdates:",
            methodParams: null,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
        clearInterval(interval);
	}
	
	Accelerometer.onUpdate = function(aX, aY, aZ) {
		if ( isLibExpired == true ) {
			return;
		}	
		this.latestAcceleration = new Acceleration(aX, aY, aZ); 
	}
	
	Accelerometer.getCurrentAcceleration = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
		if (typeof successCB == "function") {
			successCB(this.latestAcceleration);
		}
	}
}

if(!WeMo.hasLibrary("camera")) {
	WeMo.addLibrary("camera");
	
	Camera = function() {
		
	}
    
    Camera.imageSource = {
        CAMERA: 0,
        GALLERY: 1
    }
    
    Camera.imageType = {
        JPEG: 0,
        PNG: 1
    }
    
    Camera.jpegQuality = {
        SFINE: 100,
        FINE:   70,
        NORMAL: 50
    }
    
    // Initialize camera parameters with default values.
    function CameraParams() {
        this.source 	= Camera.imageSource.CAMERA,
        this.type	= Camera.imageType.JPEG,
        this.quality = Camera.jpegQuality.NORMAL
    }
	
	Camera.captureImage = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("Camera Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Camera Error: errorCallback is not a function");
            return;
        }
        
        if (typeof params === 'undefined') {
            params = new CameraParams();
        }
        
		WeMo.request = {
            className : "CameraLib",
            methodName : "captureImage:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
}

if(!WeMo.hasLibrary("contacts")) {
	WeMo.addLibrary("contacts");
    
	Contacts = function() {
	}
    
    Contacts.getContacts = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "ContactsLib",
            methodName : "getContacts:",
            methodParams: null,
            callbackId: null	
		}
        WeMo.sendRequest(successCB, errorCB);
    }
    
    Contacts.getContactDetails = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "ContactsLib",
            methodName : "getContactDetails:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Contacts.addContact = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "ContactsLib",
            methodName : "addContact:params:",
            methodParams: params,
            callbackId: null
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Contacts.editContact = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "ContactsLib",
            methodName : "editContact:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Contacts.deleteContact = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "ContactsLib",
            methodName : "deleteContact:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
}

if(!WeMo.hasLibrary("geolocation")) {
    WeMo.addLibrary("geolocation");
    
    Geolocation = function() {
        this.latitude = 0;
        this.longitude = 0;
    }
    
    Geolocation.initialize = function() {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "GeolocationLib",
            methodName : "initialize",
            methodParams: null,
            callbackId: null	
        }
        WeMo.sendRequest(null, null);
    }
    
    Geolocation.startUpdatingLocation = function() {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "GeolocationLib",
            methodName : "startUpdatingLocation",
            methodParams: null,
            callbackId: null	
        }
        WeMo.sendRequest(null, null);
    }
    
    Geolocation.stopUpdatingLocation = function() {
		if ( isLibExpired == true ) {
			return;
		}
		
        WeMo.request = {
            className : "GeolocationLib",
            methodName : "stopUpdatingLocation",
            methodParams: null,
            callbackId: null	
        }
        WeMo.sendRequest(null, null);
    }
    
    Geolocation.getResultLatitude = function(lat) {
		if ( isLibExpired == true ) {
			return;
		}
		
        this.latitude = lat; 
    }
    
    Geolocation.getResultLongitude = function(long) {
		if ( isLibExpired == true ) {
			return;
		}
		
        this.longitude = long; 
    }
    
    Geolocation.getLatitude = function() {
		if ( isLibExpired == true ) {
			return;
		}
		
        return this.latitude; 
    }
    
    Geolocation.getLongitude = function() {
		if ( isLibExpired == true ) {
			return;
		}
		
        return this.longitude; 
    }    
}

if(!WeMo.hasLibrary("file_access")) {
	WeMo.addLibrary("file_access");
	
	FileAccess = function() {
		
	};
	
	FileAccess.fileExists = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
		// successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
        if (!params.filePath) {
            console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) {
				errorCB("FileAccess Error: file is undefined!");
			}
            return;
        }
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "fileExists:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	};
	FileAccess.isDirectory = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
		// successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.filePath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file is undefined!");
			return;
		}
		
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "pathIsDirectory:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	};
	
	FileAccess.readFile = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.filePath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "readFile:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	FileAccess.writeToFile = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (typeof errorCB != "function") {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.filePath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file is undefined!");
			return;
		}
		
		if (!params.fileContents) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file contents is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "writeToFile:params:",
	        methodParams: params,
	        callbackId: null
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	FileAccess.copyFile = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.fromPath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file source is undefined!");
			return;
		}
        
		if (!params.toPath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file destination is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "copyFile:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	FileAccess.moveFile = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.fromPath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file source is undefined!");
			return;
		}
        
		if (!params.toPath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file destination is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "moveFile:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	FileAccess.deleteFile = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.filePath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "deleteFile:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	FileAccess.listPathContents = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.filePath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "listDirectoryContents:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	FileAccess.createDirectory = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.filePath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("FileAccess Error: file is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "createDir:params:",
	        methodParams: params,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
    FileAccess.getImage = function(successCB, errorCB, params) {
        // successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		if (!params.fromPath) 
		{
			console.log("FileAccess Error: file is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
                errorCB("FileAccess Error: file is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FileAccessLib",
            methodName : "getImageFromPath:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
}

if(!WeMo.hasLibrary("connectivity")) {
	WeMo.addLibrary("connectivity");
	
	Connectivity = function() {
		
	};
	
    Connectivity.getInternetSource = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		WeMo.request = {
            className : "ConnectivityLib",
            methodName : "getInternetSource:",
	        methodParams: null,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
    Connectivity.hasInternet = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("FileAccess Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("FileAccess Error: errorCallback is not a function");
            return;
        }
        
		WeMo.request = {
            className : "ConnectivityLib",
            methodName : "hasInternet:",
	        methodParams: null,
	        callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
}

if (!WeMo.hasLibrary("twitter")) {
	WeMo.addLibrary("twitter");
	
	Twitter = function() {
		
	};
	Twitter.postTweet = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("Twitter Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Twitter Error: errorCallback is not a function");
            return;
        }
        
        if (!params.tweet) 
		{
			console.log("Twitter Error: tweet is undefined!");
			if (errorCB && (typeof errorCB == "function")) 
            errorCB("Twitter Error: tweet is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "TwitterLib",
            methodName : "postTweet:params:",
            methodParams: {tweet : params.tweet},
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
	
	Twitter.getPublicTimeline = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("Twitter Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Twitter Error: errorCallback is not a function");
            return;
        }
        
		WeMo.request = {
            className : "TwitterLib",
            methodName : "getTimeline:params:",
            methodParams: {timeline : "public"},
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
	
	Twitter.getHomeTimeline = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("Twitter Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Twitter Error: errorCallback is not a function");
            return;
        }
        
		WeMo.request = {
            className : "TwitterLib",
            methodName : "getTimeline:params:",
            methodParams: {timeline : "home"},
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
	
	Twitter.getUserTimeline = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("Twitter Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Twitter Error: errorCallback is not a function");
            return;
        }
        
		WeMo.request = {
            className : "TwitterLib",
            methodName : "getTimeline:params:",
            methodParams: {timeline : "user"},
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
	
	Twitter.deleteTweet = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("Twitter Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Twitter Error: errorCallback is not a function");
            return;
        }
        
        if (typeof params.tweetId === "undefined") {
        	console.log("Twitter Error: param is not set");
        	return;
        }
        
		WeMo.request = {
            className : "TwitterLib",
            methodName : "deleteTweet:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
}

if (!WeMo.hasLibrary("facebook")) {
	WeMo.addLibrary("facebook");
	
	Facebook = function() {
		
	};
	
	// params - app_id, permissions
	Facebook.initialize = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
		// app_id is required.
		if (typeof params.app_id === "undefined") {
			console.log("Facebook Error: status is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FacebookLib",
            methodName : "initFB:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	Facebook.logout = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
		WeMo.request = {
            className : "FacebookLib",
            methodName : "logoutFB:",
            methodParams: null,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
	}
	
	// params can contain @"link", @"picture", @"name", @"caption", @"description", @"message"
	Facebook.postStatus = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("Facebook Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Facebook Error: errorCallback is not a function");
            return;
        }
        
        // message should be defined at least
        if (!params.message) {
			console.log("Facebook Error: status is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FacebookLib",
            methodName : "postStatus:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
	
	// params - path - i.e. "me/home", "me/feed"
	Facebook.queryGraph = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
    	// successCallback required
        if (typeof successCB != "function") {
            console.log("Facebook Error: successCallback is not a function");
            return;
        }
        
        // errorCallback optional
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Facebook Error: errorCallback is not a function");
            return;
        }
        
        if (!params.path) {
			console.log("Facebook Error: status is undefined!");
			return;
		}
        
		WeMo.request = {
            className : "FacebookLib",
            methodName : "queryGraph:params:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
}

if(!WeMo.hasLibrary("deviceinfo")) {
	WeMo.addLibrary("deviceinfo");
    
	DeviceInformation = function() {
	}
    
    DeviceInformation.getDeviceInfo = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("DeviceInformation Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("DeviceInformation Error: errorCallback is not a function");
            return;
        }
        
        WeMo.request = {
            className : "DeviceInfoLib",
            methodName : "getDeviceInfo:",
            methodParams: null,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
}

if(!WeMo.hasLibrary("mediaplayer")) {
	WeMo.addLibrary("mediaplayer");
    
	MediaPlayer = function() {
        
	}
    
    MediaPlayer.selectAudio = function() {
		if ( isLibExpired == true ) {
			return;
		}
        
        WeMo.request = {
            className : "MediaPlayerLib",
            methodName : "openMediaPicker",
            methodParams: null,
            callbackId: null	
		}
		WeMo.sendRequest(null, null);
    }
    
    MediaPlayer.playAudio = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Media Player Error: successCallback is not a function");
            return;
        }
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Media Player Error: errorCallback is not a function");
            return;
        }
        
        WeMo.request = {
            className : "MediaPlayerLib",
            methodName : "playMusic:",
            methodParams: null,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    MediaPlayer.pauseAudio = function(successCB, errorCB) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Media Player Error: successCallback is not a function");
            return;
        }
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Media Player Error: errorCallback is not a function");
            return;
        }
        
        WeMo.request = {
            className : "MediaPlayerLib",
            methodName : "pauseMusic:",
            methodParams: null,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
}

if(!WeMo.hasLibrary("events")) {
	WeMo.addLibrary("events");
    
	Events = function() {
        var callback;
	}
    
    Events.registerCallback = function(successCallback) {
		if ( isLibExpired == true ) {
			return;
		}
        
        if (typeof successCallback != "function") {
            console.log("Events Error: successCallback is not a function");
            return;
        }
        callback = successCallback;
    }
    
    Events.requestCallback = function(eventParam) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (!callback || typeof callback != "function") {
            return;
        }
        
        param = {"arg" : eventParam};
        WeMo.request = {
            className : "EventsLib",
            methodName : "triggeredEvents:params:",
            methodParams: param,
            callbackId: null	
		}
		WeMo.sendRequest(callback, null);
    }
}

if(!WeMo.hasLibrary("database")) {
	WeMo.addLibrary("database");
    
	Database = function() {
        var success;
	}
    
    Database.getDatabases = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
        
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (params && (typeof params == "function")) {
            console.log("Database Error: params is a function");
            return;
        }
        
        success = successCB;
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "getDatabases:",
            methodParams: params,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.createDatabase = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
        
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }   
        
        args = {"dbname" : params.dbname};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "createDatabase:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.getTables = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        } 
        
        success = successCB;
        args = {"dbname" : params.dbname};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "getTables:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.createTable = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }
        if (!params.tablename) {
            console.log("Database Error: tablename is undefined");
            return;
        } 
        var dbName = params.dbname;
        var tableName = params.tablename;
        var data = Database.createColoumnData(params.columns);
        args = {"dbname" : dbName, "tablename" : tableName, "coldetail" : data};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "createTable:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.getColumnDetails = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }
        
        if (!params.tablename) {
            console.log("Database Error: tablename is undefined");
            return;
        }
        
        success = successCB;
        args = {"dbname" : params.dbname, "tablename" : params.tablename};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "getColumnsDetail:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.insertRecord = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }
        if (!params.tablename) {
            console.log("Database Error: tablename is undefined");
            return;
        }
        if (!params.values) {
            console.log("Database Error: data is undefined");
            return;
        }
        var value = Database.createListValue(params.values);
        var type = Database.createListType(params.values);
        var name = Database.createListName(params.values);
        var dummy = Database.createDummyValue(params.values);
        args = {"dbname" : params.dbname, "tablename" : params.tablename, "colname" : name ,"colvalue" : value, "coltype" : type, "coldummy" : dummy};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "insertData:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.queryRecord = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }
        if (!params.tablename) {
            console.log("Database Error: tablename is undefined");
            return;
        }
        if (!params.columns) {
            console.log("Database Error: data is undefined");
            return;
        }
        success = successCB;
        var data = params.columns;
        var condition = params.where;
        var group = params.groupby;
        var order = params.orderby;
        if (condition == null)
        condition = "";
        if (group == null)
        group = "";
        if (order == null)
        order = "";
        
        args = {"dbname" : params.dbname, "tablename" : params.tablename, "colname" : data, "condition" : condition, "group" : group, "order" : order};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "selectData:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.deleteRecord = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }
        if (!params.tablename) {
            console.log("Database Error: tablename is undefined");
            return;
        }
        if (!params.where) {
            console.log("Database Error: \"where\" condition is undefined");
            return;
        }
        var condition = params.where;
        if (condition == null)
        condition = "";
        
        args = {"dbname" : params.dbname, "tablename" : params.tablename, "condition" : condition};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "deleteData:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.dropTable = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }
        if (!params.tablename) {
            console.log("Database Error: tablename is undefined");
            return;
        }
        var dbName = params.dbname;
        var tableName = params.tablename;
        args = {"dbname" : dbName, "tablename" : tableName};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "dropTable:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.updateRecord = function(successCB, errorCB, params) {
		if ( isLibExpired == true ) {
			return;
		}
		
        if (typeof successCB != "function") {
            console.log("Database Error: successCallback is not a function");
            return;
        }
        
        if (errorCB && (typeof errorCB != "function")) {
            console.log("Database Error: errorCallback is not a function");
            return;
        }
        
        if (!params.dbname) {
            console.log("Database Error: dbname is undefined");
            return;
        }
        if (!params.tablename) {
            console.log("Database Error: tablename is undefined");
            return;
        }
        if (!params.values) {
            console.log("Database Error: data is undefined");
            return;
        }
        if (!params.where) {
            console.log("Database Error: where condition is undefined");
            return;
        }
        var data = Database.createNewData(params.values);
        args = {"dbname" : params.dbname, "tablename" : params.tablename, "updatedetail" : data, "condition" : params.where};
        WeMo.request = {
            className : "DatabaseLib",
            methodName : "updateData:params:",
            methodParams: args,
            callbackId: null	
		}
		WeMo.sendRequest(successCB, errorCB);
    }
    
    Database.createColoumnData = function(columns) {
		if ( isLibExpired == true ) {
			return;
		}
		
        var stringCol = "";
        for (var index = 0; index < columns.length; index++) {
            stringCol += columns[index].name;
            stringCol += " ";
            stringCol += columns[index].type;
            
            if (index < columns.length - 1)
            stringCol += ",";
        }
        
        return stringCol;
    }
    
    Database.createValueData = function(entry) {
		if ( isLibExpired == true ) {
			return;
		}
		
        var stringEntry = "";
        for (var index = 0; index < entry.length; index++) {
            if (entry[index].type.toUpperCase() == "TEXT") {
                stringEntry += "'";
                stringEntry += entry[index].value;
                stringEntry += "' ";
            } else {
                stringEntry += entry[index].value;
            }
            
            if (index < entry.length - 1)
            stringEntry += ", ";
        }
        
        return stringEntry;
    }
    
    Database.createListType = function(entry) {
		if ( isLibExpired == true ) {
			return;
		}
		
        var stringEntry = "";
        for (var index = 0; index < entry.length; index++) {
            stringEntry += entry[index].type;
            
            if (index < entry.length - 1)
            stringEntry += ",";
        }
        return stringEntry;
    }
    
    Database.createListValue = function(entry) {
		if ( isLibExpired == true ) {
			return;
		}
		
        var stringEntry = "";
        for (var index = 0; index < entry.length; index++) {
            stringEntry += entry[index].value;
            
            if (index < entry.length - 1)
            stringEntry += "!@#$%^&*()_+|";
        }
        return stringEntry;
    }
    
    Database.createListName = function(entry) {
		if ( isLibExpired == true ) {
			return;
		}
		
        var stringEntry = "";
        for (var index = 0; index < entry.length; index++) {
            stringEntry += entry[index].name;
            
            if (index < entry.length - 1)
            stringEntry += ", ";
        }
        return stringEntry;
    }
    
    Database.createDummyValue = function(entry) {
		if ( isLibExpired == true ) {
			return;
		}
		
        var stringEntry = "";
        for (var index = 0; index < entry.length; index++) {
            stringEntry += "?";
            
            if (index < entry.length - 1)
            stringEntry += ", ";
        }
        return stringEntry;
    }
    
    Database.createNewData = function(entry) {
		if ( isLibExpired == true ) {
			return;
		}
		
        var stringEntry = "";
        for (var index = 0; index < entry.length; index++) {
            stringEntry += entry[index].name;
            stringEntry += " = ";
            
            if (entry[index].type.toUpperCase() == "TEXT") {
                stringEntry += "'";
                stringEntry += entry[index].value;
                stringEntry += "' ";
            } else {
                stringEntry += entry[index].value;
            }
            
            if (index < entry.length - 1)
            stringEntry += ",";
        }
        
        return stringEntry;
    }
}
